﻿namespace Infragistics.Olap.Adomd
{
    using System;
    using System.ComponentModel;

    using Infragistics.Olap;
    using Infragistics.Olap.Data;
    using Infragistics.Olap.Data.Base;
    using Infragistics.Olap.Xmla;

    public class AdomdConnectionSettings : IConnectionSettings
    {
        #region Static constructor

        /// <summary>
        /// Initializes static members of the <see cref="AdomdConnectionSettings"/> class. 
        /// Registers classes into container.
        /// </summary>
        static AdomdConnectionSettings()
        {
            OlapComposition olapComposition = new OlapComposition();
            olapComposition.AddOlapPart<AdomdDataSource>(typeof(IDataProvider));
            olapComposition.AddOlapPart<AdomdDataProvider>(typeof(IOlapConnection));
            olapComposition.AddOlapPart<AdomdConnection>(typeof(IConnectionSettings));
            olapComposition.AddOlapPart<AdomdConnectionSettings>();

            OlapContainer<IOlapViewModel>.Instance.Register(typeof(AdomdConnectionSettings), olapComposition);

            OlapComposition dataProviderComposition = new OlapComposition();
            dataProviderComposition.AddOlapPart<AdomdDataProvider>(typeof(IOlapConnection));
            dataProviderComposition.AddOlapPart<AdomdConnection>(typeof(IConnectionSettings));
            dataProviderComposition.AddOlapPart<AdomdConnectionSettings>();

            OlapContainer<IDataProvider>.Instance.Register(typeof(AdomdConnectionSettings), dataProviderComposition);

            OlapComposition filterComposition = new OlapComposition();
            filterComposition.AddOlapPart<FilterViewModelBase>(typeof(IOlapViewModel), typeof(IDataProvider), typeof(ICube), typeof(IHierarchy));
            filterComposition.AddOlapPart<AdomdDataProvider>(typeof(IOlapConnection));
            filterComposition.AddOlapPart<AdomdConnection>(typeof(IConnectionSettings));
            filterComposition.AddOlapPart<AdomdConnectionSettings>();
            filterComposition.AddOlapPart<Cube>();
            filterComposition.AddOlapPart<Hierarchy>();

            OlapContainer<IFilterViewModel>.Instance.Register(typeof(AdomdConnectionSettings), filterComposition);

            OlapComposition measureComposition = new OlapComposition();
            measureComposition.AddOlapPart<BaseMeasureViewModel>(typeof(IOlapViewModel), typeof(IDataProvider), typeof(ICube), typeof(IMeasure));
            measureComposition.AddOlapPart<AdomdDataProvider>(typeof(IOlapConnection));
            measureComposition.AddOlapPart<AdomdConnection>(typeof(IConnectionSettings));
            measureComposition.AddOlapPart<AdomdConnectionSettings>();
            measureComposition.AddOlapPart<Cube>();
            measureComposition.AddOlapPart<Measure>();

            OlapContainer<IMeasureViewModel>.Instance.Register(typeof(AdomdConnectionSettings), measureComposition);
        }

        #endregion // Static constructor
        public AdomdConnectionSettings()
        {
        }

        [TypeConverter(typeof(UriTypeConverter))]
        public Uri ServerUrl { get; set; }

        public string CatalogName { get; set; }

        public string ConnectionString { get; set; }
    }
}
