﻿using Infragistics.Olap.Xmla;
using System.Linq;
using Infragistics.Olap;
using Infragistics.Olap.Adomd.AnalysisAccess;

namespace Infragistics.Olap.Adomd
{
    /// <summary>
    /// Represents a controller that contains the data provided from model provider built over <see cref="Microsoft.AnalysisServices.AdomdClient"/>.
    /// </summary>
    public class AdomdDataSource : XmlaDataSource
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AdomdDataSource"/> class.
        /// </summary>
        /// <param name="provider">The data provider</param>
        public AdomdDataSource(IDataProvider provider)
            : base(provider)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AdomdDataSource"/> class.
        /// </summary>
        public AdomdDataSource()
        {
        }

        protected override bool UpdateSlicerReferenceState(IPivotDataSlicer target)
        {
            string commandText = this.GetSlicerStateQuery(target);
            if (string.IsNullOrEmpty(commandText))
            {
                foreach (IPivotDataSlicerItem pivotDataSlicerItem in target.Items)
                {
                    pivotDataSlicerItem.FilterSource.IsReferenceExcluded = false;
                }

                return false;
            }

            AdomdConnectorClient client = new AdomdConnectorClient();
            client.ExecuteCommandCompleted += (s, args) =>
            {
                AxisData colsAxis = args.Result.Axes[0];
                AxisData rowsAxis = args.Result.Axes[1];
                int columnsCount = colsAxis.Positions.Count;
                int rowsCount = rowsAxis.Positions.Count;

                for (int i = 0; i < rowsAxis.Positions.Count; i++)
                {
                    PositionData position = rowsAxis.Positions[i];
                    IPivotDataSlicerItem slicerItem = target.Items.
                        Where(si => si.FilterSource.Member.UniqueName == position.Members[0].UniqueName).
                        FirstOrDefault();
                    int cellIndex = i * columnsCount + 1;

                    bool isReferenceExcluded = double.Parse(args.Result.Cells[cellIndex].Value.ToString()) == 0 ? true : false;
                    this.UpdateSlicerItemReferenceState(slicerItem, isReferenceExcluded);                    
                }
            };

            client.ExecuteCommandAsync(((AdomdConnectionSettings)this.ConnectionSettings).ConnectionString, commandText);
            return true;
        }
    }
}
